#include "bilInternal.h"

static BILconvertFunction gFunctionTable[BIL_NUM_FORMATS+1][BIL_NUM_TYPES+1]
                                        [BIL_NUM_FORMATS+1][BIL_NUM_TYPES+1];

#define CONVERT_FUNCTION_NAME(SRC_FORMAT, SRC_TYPE, DST_FORMAT, DST_TYPE)         \
  bilConvert##SRC_FORMAT##SRC_TYPE##To##DST_FORMAT##DST_TYPE

#define MAKE_CONVERT_FUNCTION(SRC_FORMAT, SRC_TYPE, DST_FORMAT, DST_TYPE, CODE)   \
  static void CONVERT_FUNCTION_NAME(SRC_FORMAT, SRC_TYPE,                         \
                                    DST_FORMAT, DST_TYPE)(BILvoid       *dst,     \
                                                          const BILvoid *src,     \
                                                          BILsizei       size)    \
    {                                                                             \
    const register SRC_TYPE *srcPtr = src;                                        \
          register DST_TYPE *dstPtr = dst;                                        \
                                                                                  \
    bilCheckParameter(dst  != NULL, return);                                      \
    bilCheckParameter(src  != NULL, return);                                      \
    bilCheckParameter(size >= 0,    return);                                      \
                                                                                  \
    if (size > 0)                                                                 \
      {                                                                           \
      do                                                                          \
        {                                                                         \
        CODE;                                                                     \
        } while (--size != 0);                                                    \
      }                                                                           \
    }                                                                             \


/* (BIL_RGBA)(BIL_UNSIGNED_BYTE) to (*)(BIL_UNSIGNED_BYTE) */
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_RGB,             BILubyte, dstPtr[0] = srcPtr[0]; dstPtr[1] = srcPtr[1]; dstPtr[2] = srcPtr[2];                        dstPtr += 3; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_RGBA,            BILubyte, dstPtr[0] = srcPtr[0]; dstPtr[1] = srcPtr[1]; dstPtr[2] = srcPtr[2]; dstPtr[3] = srcPtr[3]; dstPtr += 4; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_RED,             BILubyte, dstPtr[0] = srcPtr[0];                                                                      dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_GREEN,           BILubyte, dstPtr[0] = srcPtr[1];                                                                      dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_BLUE,            BILubyte, dstPtr[0] = srcPtr[2];                                                                      dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_ALPHA,           BILubyte, dstPtr[0] = srcPtr[3];                                                                      dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_LUMINANCE,       BILubyte, dstPtr[0] = (srcPtr[0] + srcPtr[1] + srcPtr[2]) / 3;                                        dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_LUMINANCE_ALPHA, BILubyte, dstPtr[0] = (srcPtr[0] + srcPtr[1] + srcPtr[2]) / 3; dstPtr[1] = srcPtr[3];                 dstPtr += 2; srcPtr += 4)

/* (BIL_RGBA)(BIL_UNSIGNED_BYTE) to (*)(BIL_UNSIGNED_SHORT) */
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_RGB,             BILushort, dstPtr[0] = srcPtr[0] * 0xFFFF / 0xFF; dstPtr[1] = srcPtr[1] * 0xFFFF / 0xFF; dstPtr[2] = srcPtr[2] * 0xFFFF / 0xFF;                                        dstPtr += 3; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_RGBA,            BILushort, dstPtr[0] = srcPtr[0] * 0xFFFF / 0xFF; dstPtr[1] = srcPtr[1] * 0xFFFF / 0xFF; dstPtr[2] = srcPtr[2] * 0xFFFF / 0xFF; dstPtr[3] = srcPtr[3] * 0xFFFF / 0xFF; dstPtr += 4; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_RED,             BILushort, dstPtr[0] = srcPtr[0] * 0xFFFF / 0xFF;                                                                                                                      dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_GREEN,           BILushort, dstPtr[0] = srcPtr[1] * 0xFFFF / 0xFF;                                                                                                                      dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_BLUE,            BILushort, dstPtr[0] = srcPtr[2] * 0xFFFF / 0xFF;                                                                                                                      dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_ALPHA,           BILushort, dstPtr[0] = srcPtr[3] * 0xFFFF / 0xFF;                                                                                                                      dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_LUMINANCE,       BILushort, dstPtr[0] = (srcPtr[0] + srcPtr[1] + srcPtr[2]) * 0xFFFF / (0xFF*3);                                                                                        dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_LUMINANCE_ALPHA, BILushort, dstPtr[0] = (srcPtr[0] + srcPtr[1] + srcPtr[2]) * 0xFFFF / (0xFF*3); dstPtr[1] = srcPtr[3] * 0xFFFF / 0xFF;                                                 dstPtr += 2; srcPtr += 4)

/* (BIL_RGBA)(BIL_UNSIGNED_BYTE) to (*)(BIL_FLOAT) */
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_RGB,             BILfloat, dstPtr[0] = (BILfloat)srcPtr[0] / (BILfloat)0xFF; dstPtr[1] = (BILfloat)srcPtr[1] / (BILfloat)0xFF; dstPtr[2] = (BILfloat)srcPtr[2] / (BILfloat)0xFF;                                                   dstPtr += 3; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_RGBA,            BILfloat, dstPtr[0] = (BILfloat)srcPtr[0] / (BILfloat)0xFF; dstPtr[1] = (BILfloat)srcPtr[1] / (BILfloat)0xFF; dstPtr[2] = (BILfloat)srcPtr[2] / (BILfloat)0xFF; dstPtr[3] = (BILfloat)srcPtr[3] / (BILfloat)0xFF; dstPtr += 4; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_RED,             BILfloat, dstPtr[0] = (BILfloat)srcPtr[0] / (BILfloat)0xFF;                                                                                                                                                       dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_GREEN,           BILfloat, dstPtr[0] = (BILfloat)srcPtr[1] / (BILfloat)0xFF;                                                                                                                                                       dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_BLUE,            BILfloat, dstPtr[0] = (BILfloat)srcPtr[2] / (BILfloat)0xFF;                                                                                                                                                       dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_ALPHA,           BILfloat, dstPtr[0] = (BILfloat)srcPtr[3] / (BILfloat)0xFF;                                                                                                                                                       dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_LUMINANCE,       BILfloat, dstPtr[0] = (BILfloat)(srcPtr[0] + srcPtr[1] + srcPtr[2]) / (BILfloat)(0xFF*3);                                                                                                                         dstPtr += 1; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RGBA, BILubyte, BIL_LUMINANCE_ALPHA, BILfloat, dstPtr[0] = (BILfloat)(srcPtr[0] + srcPtr[1] + srcPtr[2]) / (BILfloat)(0xFF*3); dstPtr[1] = (BILfloat)srcPtr[3] / (BILfloat)0xFF;                                                                       dstPtr += 2; srcPtr += 4)

/* (*)(BIL_UNSIGNED_BYTE) to (BIL_RGBA)(BIL_UNSIGNED_BYTE) */
MAKE_CONVERT_FUNCTION(BIL_RGB,             BILubyte, BIL_RGBA, BILubyte, dstPtr[0] = srcPtr[0]; dstPtr[1] = srcPtr[1]; dstPtr[2] = srcPtr[2]; dstPtr[3] = 0xFF;      dstPtr += 4; srcPtr += 3)
MAKE_CONVERT_FUNCTION(BIL_RED,             BILubyte, BIL_RGBA, BILubyte, dstPtr[0] = srcPtr[0]; dstPtr[1] = 0x00;      dstPtr[2] = 0x00;      dstPtr[3] = 0xFF;      dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_GREEN,           BILubyte, BIL_RGBA, BILubyte, dstPtr[0] = 0x00;      dstPtr[1] = srcPtr[0]; dstPtr[2] = 0x00;      dstPtr[3] = 0xFF;      dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_BLUE,            BILubyte, BIL_RGBA, BILubyte, dstPtr[0] = 0x00;      dstPtr[1] = 0x00;      dstPtr[2] = srcPtr[0]; dstPtr[3] = 0xFF;      dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_ALPHA,           BILubyte, BIL_RGBA, BILubyte, dstPtr[0] = 0x00;      dstPtr[1] = 0x00;      dstPtr[2] = 0x00;    ; dstPtr[3] = srcPtr[0]; dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_LUMINANCE,       BILubyte, BIL_RGBA, BILubyte, dstPtr[0] = srcPtr[0]; dstPtr[1] = srcPtr[0]; dstPtr[2] = srcPtr[0]; dstPtr[3] = 0xFF;      dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_LUMINANCE_ALPHA, BILubyte, BIL_RGBA, BILubyte, dstPtr[0] = srcPtr[0]; dstPtr[1] = srcPtr[0]; dstPtr[2] = srcPtr[0]; dstPtr[3] = srcPtr[1]; dstPtr += 4; srcPtr += 2)

/* (*)(BIL_UNSIGNED_SHORT) to (BIL_RGBA)(BIL_UNSIGNED_BYTE) */
MAKE_CONVERT_FUNCTION(BIL_RGB,             BILushort, BIL_RGBA, BILubyte, dstPtr[0] = srcPtr[0] * 0xFF / 0xFFFF; dstPtr[1] = srcPtr[1] * 0xFF / 0xFFFF; dstPtr[2] = srcPtr[2] * 0xFF / 0xFFFF; dstPtr[3] = 0xFF;                      dstPtr += 4; srcPtr += 3)
MAKE_CONVERT_FUNCTION(BIL_RGBA,            BILushort, BIL_RGBA, BILubyte, dstPtr[0] = srcPtr[0] * 0xFF / 0xFFFF; dstPtr[1] = srcPtr[1] * 0xFF / 0xFFFF; dstPtr[2] = srcPtr[2] * 0xFF / 0xFFFF; dstPtr[3] = srcPtr[3] * 0xFF / 0xFFFF; dstPtr += 4; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RED,             BILushort, BIL_RGBA, BILubyte, dstPtr[0] = srcPtr[0] * 0xFF / 0xFFFF; dstPtr[1] = 0x00;                      dstPtr[2] = 0x00;                      dstPtr[3] = 0xFF;                      dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_GREEN,           BILushort, BIL_RGBA, BILubyte, dstPtr[0] = 0x00;                      dstPtr[1] = srcPtr[0] * 0xFF / 0xFFFF; dstPtr[2] = 0x00;                      dstPtr[3] = 0xFF;                      dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_BLUE,            BILushort, BIL_RGBA, BILubyte, dstPtr[0] = 0x00;                      dstPtr[1] = 0x00;                      dstPtr[2] = srcPtr[0] * 0xFF / 0xFFFF; dstPtr[3] = 0xFF;                      dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_ALPHA,           BILushort, BIL_RGBA, BILubyte, dstPtr[0] = 0x00;                      dstPtr[1] = 0x00;                      dstPtr[2] = 0x00;                      dstPtr[3] = srcPtr[0] * 0xFF / 0xFFFF; dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_LUMINANCE,       BILushort, BIL_RGBA, BILubyte, dstPtr[0] = srcPtr[0] * 0xFF / 0xFFFF; dstPtr[1] = srcPtr[0] * 0xFF / 0xFFFF; dstPtr[2] = srcPtr[0] * 0xFF / 0xFFFF; dstPtr[3] = 0xFF;                      dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_LUMINANCE_ALPHA, BILushort, BIL_RGBA, BILubyte, dstPtr[0] = srcPtr[0] * 0xFF / 0xFFFF; dstPtr[1] = srcPtr[0] * 0xFF / 0xFFFF; dstPtr[2] = srcPtr[0] * 0xFF / 0xFFFF; dstPtr[3] = srcPtr[1] * 0xFF / 0xFFFF; dstPtr += 4; srcPtr += 2)

/* (*)(BIL_FLOAT) to (BIL_RGBA)(BIL_UNSIGNED_BYTE) */
MAKE_CONVERT_FUNCTION(BIL_RGB,             BILfloat, BIL_RGBA, BILubyte, dstPtr[0] = (BILubyte)(srcPtr[0] * (BILfloat)0xFF); dstPtr[1] = (BILubyte)(srcPtr[1] * (BILfloat)0xFF); dstPtr[2] = (BILubyte)(srcPtr[2] * (BILfloat)0xFF); dstPtr[3] = 0xFF;                                   dstPtr += 4; srcPtr += 3)
MAKE_CONVERT_FUNCTION(BIL_RGBA,            BILfloat, BIL_RGBA, BILubyte, dstPtr[0] = (BILubyte)(srcPtr[0] * (BILfloat)0xFF); dstPtr[1] = (BILubyte)(srcPtr[1] * (BILfloat)0xFF); dstPtr[2] = (BILubyte)(srcPtr[2] * (BILfloat)0xFF); dstPtr[3] = (BILubyte)(srcPtr[3] * (BILfloat)0xFF); dstPtr += 4; srcPtr += 4)
MAKE_CONVERT_FUNCTION(BIL_RED,             BILfloat, BIL_RGBA, BILubyte, dstPtr[0] = (BILubyte)(srcPtr[0] * (BILfloat)0xFF); dstPtr[1] = 0x00;                                   dstPtr[2] = 0x00;                                   dstPtr[3] = 0xFF;                                   dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_GREEN,           BILfloat, BIL_RGBA, BILubyte, dstPtr[0] = 0x00;                                   dstPtr[1] = (BILubyte)(srcPtr[0] * (BILfloat)0xFF); dstPtr[2] = 0x00;                                   dstPtr[3] = 0xFF;                                   dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_BLUE,            BILfloat, BIL_RGBA, BILubyte, dstPtr[0] = 0x00;                                   dstPtr[1] = 0x00;                                   dstPtr[2] = (BILubyte)(srcPtr[0] * (BILfloat)0xFF); dstPtr[3] = 0xFF;                                   dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_ALPHA,           BILfloat, BIL_RGBA, BILubyte, dstPtr[0] = 0x00;                                   dstPtr[1] = 0x00;                                   dstPtr[2] = 0x00;                                   dstPtr[3] = (BILubyte)(srcPtr[0] * (BILfloat)0xFF); dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_LUMINANCE,       BILfloat, BIL_RGBA, BILubyte, dstPtr[0] = (BILubyte)(srcPtr[0] * (BILfloat)0xFF); dstPtr[1] = (BILubyte)(srcPtr[0] * (BILfloat)0xFF); dstPtr[2] = (BILubyte)(srcPtr[0] * (BILfloat)0xFF); dstPtr[3] = 0xFF;                                   dstPtr += 4; srcPtr += 1)
MAKE_CONVERT_FUNCTION(BIL_LUMINANCE_ALPHA, BILfloat, BIL_RGBA, BILubyte, dstPtr[0] = (BILubyte)(srcPtr[0] * (BILfloat)0xFF); dstPtr[1] = (BILubyte)(srcPtr[0] * (BILfloat)0xFF); dstPtr[2] = (BILubyte)(srcPtr[0] * (BILfloat)0xFF); dstPtr[3] = (BILubyte)(srcPtr[1] * (BILfloat)0xFF); dstPtr += 4; srcPtr += 2)


inline static BILint bilFormatIndex(BILformat format)
  {
  switch (format)
    {
    case BIL_RGB:             return 1;
    case BIL_RGBA:            return 2;
    case BIL_RED:             return 3;
    case BIL_GREEN:           return 4;
    case BIL_BLUE:            return 5;
    case BIL_ALPHA:           return 6;
    case BIL_LUMINANCE:       return 7;
    case BIL_LUMINANCE_ALPHA: return 8;
    default:                  assert(0);
    }

  bilSetError(BIL_INVALID_ENUM);
  return 0;
  }


inline static BILint bilTypeIndex(BILtype type)
  {
  switch (type)
    {    
    case BIL_UNSIGNED_BYTE:  return 1;
    case BIL_UNSIGNED_SHORT: return 2;
    case BIL_FLOAT:          return 3;
    default:                 assert(0);
    }

  bilSetError(BIL_INVALID_ENUM);
  return 0;
  }


static void bilBuildFunctionTable()
  {
  static BILboolean gTableBuilt = BIL_FALSE;

  if (!gTableBuilt)
    {
    gTableBuilt = BIL_TRUE;

    /* fill table with blanks */
    memset(gFunctionTable, 0, sizeof(gFunctionTable));

    /* (BIL_RGBA)(BIL_UNSIGNED_BYTE) to (*)(BIL_UNSIGNED_BYTE) */
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGB            )][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_RGB,             BILubyte);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGBA           )][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_RGBA,            BILubyte);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RED            )][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_RED,             BILubyte);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_GREEN          )][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_GREEN,           BILubyte);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_BLUE           )][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_BLUE,            BILubyte);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_ALPHA          )][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_ALPHA,           BILubyte);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_LUMINANCE      )][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_LUMINANCE,       BILubyte);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_LUMINANCE_ALPHA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_LUMINANCE_ALPHA, BILubyte);

    /* (BIL_RGBA)(BIL_UNSIGNED_BYTE) to (*)(BIL_UNSIGNED_SHORT) */
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGB            )][bilTypeIndex(BIL_UNSIGNED_SHORT)] = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_RGB,             BILushort);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGBA           )][bilTypeIndex(BIL_UNSIGNED_SHORT)] = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_RGBA,            BILushort);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RED            )][bilTypeIndex(BIL_UNSIGNED_SHORT)] = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_RED,             BILushort);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_GREEN          )][bilTypeIndex(BIL_UNSIGNED_SHORT)] = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_GREEN,           BILushort);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_BLUE           )][bilTypeIndex(BIL_UNSIGNED_SHORT)] = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_BLUE,            BILushort);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_ALPHA          )][bilTypeIndex(BIL_UNSIGNED_SHORT)] = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_ALPHA,           BILushort);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_LUMINANCE      )][bilTypeIndex(BIL_UNSIGNED_SHORT)] = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_LUMINANCE,       BILushort);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_LUMINANCE_ALPHA)][bilTypeIndex(BIL_UNSIGNED_SHORT)] = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_LUMINANCE_ALPHA, BILushort);

    /* (BIL_RGBA)(BIL_UNSIGNED_BYTE) to (*)(BIL_FLOAT) */
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGB            )][bilTypeIndex(BIL_FLOAT)]          = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_RGB,             BILfloat);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGBA           )][bilTypeIndex(BIL_FLOAT)]          = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_RGBA,            BILfloat);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RED            )][bilTypeIndex(BIL_FLOAT)]          = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_RED,             BILfloat);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_GREEN          )][bilTypeIndex(BIL_FLOAT)]          = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_GREEN,           BILfloat);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_BLUE           )][bilTypeIndex(BIL_FLOAT)]          = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_BLUE,            BILfloat);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_ALPHA          )][bilTypeIndex(BIL_FLOAT)]          = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_ALPHA,           BILfloat);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_LUMINANCE      )][bilTypeIndex(BIL_FLOAT)]          = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_LUMINANCE,       BILfloat);
    gFunctionTable[bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_LUMINANCE_ALPHA)][bilTypeIndex(BIL_FLOAT)]          = CONVERT_FUNCTION_NAME(BIL_RGBA, BILubyte, BIL_LUMINANCE_ALPHA, BILfloat);

    /* (*)(BIL_UNSIGNED_BYTE) to (BIL_RGBA)(BIL_UNSIGNED_BYTE) */
    gFunctionTable[bilFormatIndex(BIL_RGB            )][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RGB,             BILubyte,   BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_RGBA           )][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RGBA,            BILubyte,   BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_RED            )][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RED,             BILubyte,   BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_GREEN          )][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_GREEN,           BILubyte,   BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_BLUE           )][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_BLUE,            BILubyte,   BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_ALPHA          )][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_ALPHA,           BILubyte,   BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_LUMINANCE      )][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_LUMINANCE,       BILubyte,   BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_LUMINANCE_ALPHA)][bilTypeIndex(BIL_UNSIGNED_BYTE)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_LUMINANCE_ALPHA, BILubyte,   BIL_RGBA, BILubyte);

    /* (*)(BIL_UNSIGNED_SHORT) to (BIL_RGBA)(BIL_UNSIGNED_BYTE) */
    gFunctionTable[bilFormatIndex(BIL_RGB            )][bilTypeIndex(BIL_UNSIGNED_SHORT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RGB,             BILushort, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_RGBA           )][bilTypeIndex(BIL_UNSIGNED_SHORT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RGBA,            BILushort, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_RED            )][bilTypeIndex(BIL_UNSIGNED_SHORT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_RED,             BILushort, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_GREEN          )][bilTypeIndex(BIL_UNSIGNED_SHORT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_GREEN,           BILushort, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_BLUE           )][bilTypeIndex(BIL_UNSIGNED_SHORT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_BLUE,            BILushort, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_ALPHA          )][bilTypeIndex(BIL_UNSIGNED_SHORT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_ALPHA,           BILushort, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_LUMINANCE      )][bilTypeIndex(BIL_UNSIGNED_SHORT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_LUMINANCE,       BILushort, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_LUMINANCE_ALPHA)][bilTypeIndex(BIL_UNSIGNED_SHORT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]  = CONVERT_FUNCTION_NAME(BIL_LUMINANCE_ALPHA, BILushort, BIL_RGBA, BILubyte);

    /* (*)(BIL_FLOAT) to (BIL_RGBA)(BIL_UNSIGNED_BYTE) */
    gFunctionTable[bilFormatIndex(BIL_RGB            )][bilTypeIndex(BIL_FLOAT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]           = CONVERT_FUNCTION_NAME(BIL_RGB,             BILfloat, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_RGBA           )][bilTypeIndex(BIL_FLOAT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]           = CONVERT_FUNCTION_NAME(BIL_RGBA,            BILfloat, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_RED            )][bilTypeIndex(BIL_FLOAT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]           = CONVERT_FUNCTION_NAME(BIL_RED,             BILfloat, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_GREEN          )][bilTypeIndex(BIL_FLOAT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]           = CONVERT_FUNCTION_NAME(BIL_GREEN,           BILfloat, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_BLUE           )][bilTypeIndex(BIL_FLOAT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]           = CONVERT_FUNCTION_NAME(BIL_BLUE,            BILfloat, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_ALPHA          )][bilTypeIndex(BIL_FLOAT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]           = CONVERT_FUNCTION_NAME(BIL_ALPHA,           BILfloat, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_LUMINANCE      )][bilTypeIndex(BIL_FLOAT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]           = CONVERT_FUNCTION_NAME(BIL_LUMINANCE,       BILfloat, BIL_RGBA, BILubyte);
    gFunctionTable[bilFormatIndex(BIL_LUMINANCE_ALPHA)][bilTypeIndex(BIL_FLOAT)][bilFormatIndex(BIL_RGBA)][bilTypeIndex(BIL_UNSIGNED_BYTE)]           = CONVERT_FUNCTION_NAME(BIL_LUMINANCE_ALPHA, BILfloat, BIL_RGBA, BILubyte);
    }
  }


BILconvertFunction bilGetConvertFunction(BILformat srcFormat, BILtype srcType,
                                         BILformat dstFormat, BILtype dstType)
  {
  BILconvertFunction function;

  bilBuildFunctionTable();

  function = gFunctionTable[bilFormatIndex(srcFormat)][bilTypeIndex(srcType)]
                           [bilFormatIndex(dstFormat)][bilTypeIndex(dstType)];
  if (!function)
    {
    bilSetError(BIL_INVALID_ENUM);
    assert(0);
    }

  return function;
  }

