#include "bilInternal.h"


BILcontext bilBeginDecode(BILimage _image, const BILchar *key)
  {
  BILimageStruct         *image = _image;
  BILdecodeContextStruct *context;
  const BILdecoderStruct *decoder;
  
  bilCheckParameter(image            != NULL,                return NULL);
  bilCheckParameter(image->signature == BIL_IMAGE_SIGNATURE, return NULL);

  /* find decoder */
  decoder = bilGetDecoder(key);
  if (!decoder)
    {
    bilSetError(BIL_INVALID_VALUE);
    return NULL;
    }

  /* make context */
  context = decoder->newDecodeContext();
  if (!context)
    return NULL;

  /* fill in our own context fields */
  context->signature = BIL_DECODE_SIGNATURE;
  context->decoder   = decoder;
  context->image     = image;

  /* start encoding */
  if (!decoder->beginDecode(context))
    {
    decoder->deleteDecodeContext(context);
    return NULL;
    }

  return context;
  }


BILsizei bilDecodeBlock(BILcontext _context, const BILvoid *block, BILsizei size)
  {
  BILdecodeContextStruct *context = _context;

  bilCheckParameter(context            != NULL,                 return 0);
  bilCheckParameter(block              != NULL,                 return 0);
  bilCheckParameter(size               >= 0,                    return 0);
  bilCheckParameter(context->signature == BIL_DECODE_SIGNATURE, return 0);

  return context->decoder->decodeBlock(context, block, size);
  }


BILboolean bilValidBlock(BILcontext _context, const BILvoid *block, BILsizei size)
  {
  BILdecodeContextStruct *context = _context;

  bilCheckParameter(context            != NULL,                 return 0);
  bilCheckParameter(block              != NULL,                 return 0);
  bilCheckParameter(size               >= 0,                    return 0);
  bilCheckParameter(context->signature == BIL_DECODE_SIGNATURE, return 0);

  return context->decoder->validBlock(context, block, size);
  }


BILboolean bilEndDecode(BILcontext _context)
  {
  BILdecodeContextStruct *context = _context;
  BILboolean              result;

  bilCheckParameter(context            != NULL,                 return 0);
  bilCheckParameter(context->signature == BIL_DECODE_SIGNATURE, return 0);

  result  = context->decoder->endDecode(context);
  result &= context->decoder->deleteDecodeContext(context);

  return result;
  }

