#include "bilInternal.h"


BILcontext bilBeginEncode(BILimage _image, const BILchar *key)
  {
  BILimageStruct          *image = _image;
  BILencodeContextStruct  *context;
  const BILencoderStruct  *encoder;
  
  bilCheckParameter(image            != NULL,                return NULL);
  bilCheckParameter(image->signature == BIL_IMAGE_SIGNATURE, return NULL);

  /* find encoder */
  encoder = bilGetEncoder(key);
  if (!encoder)
    {
    bilSetError(BIL_INVALID_VALUE);
    return NULL;
    }

  /* make context */
  context = encoder->newEncodeContext();
  if (!context)
    return NULL;

  /* fill in our own context fields */
  context->signature = BIL_ENCODE_SIGNATURE;
  context->encoder   = encoder;
  context->image     = image;

  /* start encoding */
  if (!encoder->beginEncode(context))
    {
    encoder->deleteEncodeContext(context);
    return NULL;
    }

  return context;
  }


BILsizei bilEncodeBlock(BILcontext _context, BILvoid *block, BILsizei size)
  {
  BILencodeContextStruct *context = _context;

  bilCheckParameter(context            != NULL,                 return 0);
  bilCheckParameter(block              != NULL,                 return 0);
  bilCheckParameter(size               >= 0,                    return 0);
  bilCheckParameter(context->signature == BIL_ENCODE_SIGNATURE, return 0);

  return context->encoder->encodeBlock(context, block, size);
  }


BILboolean bilEndEncode(BILcontext _context)
  {
  BILencodeContextStruct *context = _context;
  BILboolean              result;

  bilCheckParameter(context            != NULL,                 return BIL_FALSE);
  bilCheckParameter(context->signature == BIL_ENCODE_SIGNATURE, return BIL_FALSE);

  /* finish encoding */
  result  = context->encoder->endEncode(context);

  /* clear signature (for debugging) */
  context->signature = 0;

  /* delete the context */
  result &= context->encoder->deleteEncodeContext(context);

  return result;
  }

