#include "bilInternal.h"


BILboolean bilSaveImage(BILimage *image, const BILchar *fileName, const BILchar *driver)
  {
  FILE       *file;
  BILcontext *context;
  BILsizei    size;
  BILubyte    buffer[8192];

  file = fopen(fileName, "wb");
  if (!file)
    {
    bilSetError(BIL_FILE_WRITE_ERROR);
    return BIL_FALSE;
    }

  /* start encoding */
  context = bilBeginEncode(image, driver);
  if (!context)
    {
    fclose(file);
    return BIL_FALSE;
    }

  /* encode blocks */
  while ((size = bilEncodeBlock(context, buffer, sizeof(buffer))) > 0)
    fwrite(buffer, size, 1, file);

  /* finish encoding */
  if (!bilEndEncode(context))
    {
    fclose(file);
    return BIL_FALSE;
    }

  /* close file */
  if (fclose(file) != 0)
    {
    bilSetError(BIL_FILE_WRITE_ERROR);
    return BIL_FALSE;
    }

  return BIL_TRUE;
  }


BILboolean bilLoadImage(BILimage *image, const BILchar *fileName, const BILchar *driver)
  {
  FILE       *file;
  BILcontext *context;
  BILsizei    size;
  BILubyte    buffer[8192];

  file = fopen(fileName, "rb");
  if (!file)
    {
    bilSetError(BIL_FILE_READ_ERROR);
    return BIL_FALSE;
    }

  /* start encoding */
  context = bilBeginDecode(image, driver);
  if (!context)
    {
    fclose(file);
    return BIL_FALSE;
    }

  /* decode blocks */
  while ((size = fread(&buffer, 1, sizeof(buffer), file)) > 0)
    bilDecodeBlock(context, &buffer, size);
  
  /* finish decoding */
  if (!bilEndDecode(context))
    {
    fclose(file);
    return BIL_FALSE;
    }

  /* close file */
  if (fclose(file) != 0)
    {
    bilSetError(BIL_FILE_READ_ERROR);
    return BIL_FALSE;
    }

  return BIL_TRUE;
  }
