#include "bilInternal.h"


static void swapRows(BILimageStruct *image, BILint topRow, BILint bottomRow,
                     BILvoid *tempRow, BILsizei rowBytes)
  {
  BILvoid  *topPixels    = image->pixels + (topRow    * image->width);
  BILvoid  *bottomPixels = image->pixels + (bottomRow * image->width);

  memcpy(tempRow,      topPixels,    rowBytes);
  memcpy(topPixels,    bottomPixels, rowBytes);
  memcpy(bottomPixels, tempRow,      rowBytes);
  }


BILboolean bilFlipImageHorizontal(BILimage _image)
  {
  BILimageStruct *image = _image;

  bilCheckParameter(image            != NULL,                return BIL_FALSE);
  bilCheckParameter(image->signature == BIL_IMAGE_SIGNATURE, return BIL_FALSE);

  /* not yet implemented */
  return BIL_FALSE;
  }


BILboolean bilFlipImageVertical(BILimage _image)
  {
  BILimageStruct *image = _image;
  BILubyte       *tempRow;
  BILsizei        rowBytes;
  BILint          topRow;
  BILint          bottomRow;

  bilCheckParameter(image            != NULL,                return BIL_FALSE);
  bilCheckParameter(image->signature == BIL_IMAGE_SIGNATURE, return BIL_FALSE);

  /* calc bytes per row */
  rowBytes = image->width * sizeof(BILpixelStruct);

  /* allocate row buffer */
  tempRow = bilMalloc(rowBytes);
  if (!tempRow)
    return BIL_FALSE;

  /* flip it... */
  topRow    = 0;
  bottomRow = image->height-1;
  while (topRow < bottomRow)
    {
    swapRows(image, topRow, bottomRow, tempRow, rowBytes);
    topRow++;
    bottomRow--;
    }

  /* free scanline buffer */
  bilFree(tempRow);

  return BIL_TRUE;
  }
