#include "bilInternal.h"

#define NULL_VALUE 0xCCCCCCCC
static BILuint null = NULL_VALUE;


BILvoid *bilMalloc(BILsizei size)
  {
  BILvoid *result;

  bilCheckParameter(size >= 0, return NULL);

  if (size == 0)
    result = &null;
  else
    {
    result = malloc((size_t)size);
    if (!result)
      bilSetError(BIL_OUT_OF_MEMORY);
    }

  return result;
  }


BILvoid bilFree(BILvoid *pointer)
  {
  if (pointer != NULL)
    {
    if (pointer == &null)
      bilAssert(null == NULL_VALUE, null = NULL_VALUE);
    else
      free(pointer);
    }
  }


BILvoid *bilRealloc(BILvoid *pointer, BILsizei size)
  {
  BILvoid *result;

  bilCheckParameter(size >= 0, return NULL);

  if ((pointer == NULL) || (pointer == &null))
    result = bilMalloc(size);

  else if (size == 0)
    {
    bilFree(pointer);
    result = &null;
    }

  else
    {
    result = realloc(pointer, (size_t)size);
    if (!result)
      bilSetError(BIL_OUT_OF_MEMORY);
    }

  return result;
  }


BILvoid *bilMemdup(const BILvoid *pointer, BILsizei size)
  {
  BILvoid *result;

  bilCheckParameter(pointer != NULL, return NULL);
  bilCheckParameter(size    >= 0,    return NULL);

  result = bilMalloc(size);
  if (result)
    memcpy(result, pointer, size);

  return result;
  }
