#include "bilInternal.h"


const BILchar *bilNextKey(const BILchar *string, BILchar *key, BILsizei keySize)
  {
  bilCheckParameter(string  != NULL, return NULL);
  bilCheckParameter(key     != NULL, return NULL);
  bilCheckParameter(keySize >= 0,    return NULL);

  /* skip leading spaces */
  while ((*string == ' ') && (*string != '\0'))
    string++;

  /* do we have a string? */
  if (*string == '\0')
    return NULL;

  /* copy all characters (except spaces) */
  while ((*string != ' ') && (*string != '\0') && (--keySize > 0))
    *key++ = *string++;

  /* append end-of-string */
  if (keySize >= 0)
    *key = '\0';

  /* skip remaining characters (except spaces) */
  while ((*string != ' ') && (*string != '\0'))
    string++;

  return string;
  }


BILboolean bilContainsKey(const BILchar *string, const BILchar *key)
  {
  BILchar    *start;
  BILchar    *end;
  BILboolean  startOK = BIL_FALSE;
  BILboolean  endOK   = BIL_FALSE;

  bilCheckParameter(string != NULL, return BIL_FALSE);
  bilCheckParameter(key    != NULL, return BIL_FALSE);

  start = strstr(string, key);
  if (!start)
    return BIL_FALSE;

  end     = start + strlen(key);
  startOK = (start == string) || (start[-1] == ' ');
  endOK   = (*end  == '\0')   || (*end      == ' ');

  return startOK && endOK;
  }

