#if defined(BIL_DLL_EXPORT)

#include <windows.h>
#include <io.h>
#include "bilInternal.h"


BILboolean bilRegisterDriver(const BILchar *fileName)
  {
  HMODULE                   library;
  BILregisterDriverFunction registerDriver;

  bilCheckParameter(fileName != NULL, return BIL_FALSE);

  /* load library */
  library = LoadLibrary(fileName);
  if (!library)
    {
    bilSetError(BIL_INVALID_OPERATION);
    return BIL_FALSE;
    }

  /* does it have the registration function? */
  registerDriver = (BILregisterDriverFunction)GetProcAddress(library, "_bilRegisterDriver");
  if (!registerDriver)
    {
    FreeLibrary(library);
    bilSetError(BIL_INVALID_OPERATION);
    return BIL_FALSE;
    }

  /* register it... */
  if (!registerDriver())
    {
    FreeLibrary(library);
    return BIL_FALSE;
    }
 
  return BIL_TRUE;
  }


BILboolean bilRegisterAllDrivers(const BILchar *path)
  {  
  char               fileName[MAX_PATH];
  long               handle;
  struct _finddata_t findData;
  BILboolean         result;

  bilCheckParameter(path != NULL, return BIL_FALSE);

  strcpy(fileName, path);
  strcat(fileName, "/*.*");

  result = BIL_FALSE;
  handle = _findfirst(fileName, &findData);
  if (handle >= 0)
    {
    do
      {
      strcpy(fileName, path);
      strcat(fileName, "/");
      strcat(fileName, findData.name);

      result |= bilRegisterDriver(fileName);
      } while (_findnext(handle, &findData) == 0);
    }

  bilSetError(BIL_NO_ERROR);
  return result;
  }

#endif

