#include "bilPpmInternal.h"


static BILcontext bilPpmNewDecodeContext()
  {
  BILPPMdecodeContextStruct *context;

  context = bilMalloc(sizeof(BILPPMdecodeContextStruct));
  if (!context)
    return NULL;

  return (BILcontext)context;
  }


static BILboolean bilPpmDeleteDecodeContext(BILcontext _context)
  {
  BILPPMdecodeContextStruct *context = _context;

  bilCheckParameter(context != NULL, return BIL_FALSE);

  bilFree(context);

  return BIL_TRUE;
  }


static BILboolean bilPpmBeginDecode(BILcontext _context)
  {
  BILPPMdecodeContextStruct *context = _context;

  bilCheckParameter(context != NULL, return BIL_FALSE);

  /* begin decoding */  
  context->tempFile = tmpfile();
  if (!context->tempFile)
    {
    bilSetError(BIL_FILE_WRITE_ERROR);
    return BIL_FALSE;
    }

  return BIL_TRUE;
  }


static BILboolean bilPpmEndDecode(BILcontext _context)
  {
  BILPPMdecodeContextStruct *context = _context;

  bilCheckParameter(context               != NULL, return BIL_FALSE);
  bilCheckParameter(context->parent.image != NULL, return BIL_FALSE);
  bilCheckParameter(context->tempFile     != NULL, return BIL_FALSE);

  /* rewind file */
  rewind(context->tempFile);

  /* do it... */
  if (!ppm_load_image(context))
    return BIL_FALSE;

  /* close files */
  if (fclose(context->tempFile) < 0)
    {
    bilSetError(BIL_FILE_WRITE_ERROR);
    return BIL_FALSE;
    }

  return BIL_TRUE;
  }


static BILsizei bilPpmDecodeBlock(BILcontext _context, const BILvoid *block, BILsizei size)
  {
  BILPPMdecodeContextStruct *context = _context;
  BILsizei                   numWritten;

  bilCheckParameter(context != NULL, return 0);
  bilCheckParameter(block   != NULL, return 0);
  bilCheckParameter(size    >= 0,    return 0);

  /* write block to temporary file */
  numWritten = fwrite(block, 1, size, context->tempFile);
  if (numWritten != size)
    {
    bilSetError(BIL_FILE_WRITE_ERROR);
    return 0;
    }

  return numWritten;
  }


static BILboolean bilPpmValidBlock(BILcontext _context, const BILvoid *block, BILsizei size)
  {
  BILPPMdecodeContextStruct *context = _context;

  bilCheckParameter(context != NULL, return BIL_FALSE);
  bilCheckParameter(block   != NULL, return BIL_FALSE);
  bilCheckParameter(size    >= 0,    return BIL_FALSE);

  /* a zero sized block is NOT valid */
  if (size <= 0)
    return BIL_FALSE;

  return BIL_TRUE;
  }


BILboolean bilPpmRegisterDecoder()
  {
  static BILparameterStruct parameters[] =
        {
        { NULL, 0, 0 }
        };

  static BILdecoderStruct decoder = 
       	{
        /* bilVersion          */ BIL_VERSION,
        /* version             */ BILPPM_VERSION,
        /* parameters          */ parameters,
        /* newDecodeContext    */ bilPpmNewDecodeContext,
        /* deleteDecodeContext */ bilPpmDeleteDecodeContext,
        /* beginDecode         */ bilPpmBeginDecode,
        /* endDecode           */ bilPpmEndDecode,
        /* decodeBlock         */ bilPpmDecodeBlock,
        /* validBlock          */ bilPpmValidBlock
        };

  return bilRegisterDecoder(BILPPM_NAME, &decoder);
  }


BILboolean bilPpmUnregisterDecoder()
  {
  return bilUnregisterDecoder(BILPPM_NAME);
  }

