#include "bilPpmInternal.h"


static BILcontext bilPpmNewEncodeContext()
  {
  BILPPMencodeContextStruct *context;

  context = bilMalloc(sizeof(BILPPMencodeContextStruct));
  if (!context)
    return NULL;

  return (BILcontext)context;
  }


static BILboolean bilPpmDeleteEncodeContext(BILcontext _context)
  {
  BILPPMencodeContextStruct *context = _context;

  bilCheckParameter(context != NULL, return BIL_FALSE);

  bilFree(context);

  return BIL_TRUE;
  }


static BILboolean bilPpmBeginEncode(BILcontext _context)
  {
  BILPPMencodeContextStruct *context = _context;
  BILboolean                 result  = BIL_FALSE;
  BILsizei                   width;
  BILsizei                   height;
  
  bilCheckParameter(context               != NULL, return BIL_FALSE);
  bilCheckParameter(context->parent.image != NULL, return BIL_FALSE);

  width  = bilGetImageWidth(context->parent.image);
  height = bilGetImageHeight(context->parent.image);

  /* create temporary file */
  context->tempFile = tmpfile();
  if (!context->tempFile)
    {
    bilSetError(BIL_FILE_WRITE_ERROR);
    goto error_end;
    }

  /* do it... */
  if (!ppm_save_image(context))
    goto error_end;

  /* set some more details */
  context->bytesToEncode = ftell(context->tempFile);
  context->bytesEncoded  = 0;

  /* rewind file */
  rewind(context->tempFile);

  /* we were successful */
  result = BIL_TRUE;

error_end:

  return result;
  }


static BILboolean bilPpmEndEncode(BILcontext _context)
  {
  BILPPMencodeContextStruct *context = _context;

  bilCheckParameter(context != NULL, return BIL_FALSE);

  /* close file */
  if (fclose(context->tempFile) < 0)
    {
    bilSetError(BIL_FILE_READ_ERROR);
    return BIL_FALSE;
    }

  /* test to make sure entire image was encoded */
  if (context->bytesEncoded != context->bytesToEncode)
    {
    bilSetError(BIL_INVALID_OPERATION);
    return BIL_FALSE;
    }

  return BIL_TRUE;
  }


static BILsizei bilPpmEncodeBlock(BILcontext _context, BILvoid *block, BILsizei size)
  {
  BILPPMencodeContextStruct *context = _context;
  BILsizei                   numRead;

  bilCheckParameter(context != NULL, return 0);
  bilCheckParameter(block   != NULL, return 0);
  bilCheckParameter(size    >= 0,    return 0);

  /* read next block */
  numRead = fread(block, 1, size, context->tempFile);

  /* check for read error */
  if (ferror(context->tempFile))
    {
    bilSetError(BIL_FILE_READ_ERROR);
    return 0;
    }

  /* increment encoding counter */
  context->bytesEncoded += numRead;

  return numRead;
  }


BILboolean bilPpmRegisterEncoder()
  {
  static BILparameterStruct parameters[] =
        {
        { NULL, 0, 0 }
        };

  static BILencoderStruct encoder = 
        {
        /* bilVersion          */ BIL_VERSION,
        /* version             */ BILPPM_VERSION,
        /* paramters           */ parameters,
        /* newEncodeContext    */ bilPpmNewEncodeContext,
        /* deleteEncodeContext */ bilPpmDeleteEncodeContext,
        /* beginEncode         */ bilPpmBeginEncode,
        /* endEncode           */ bilPpmEndEncode,
        /* encodeBlock         */ bilPpmEncodeBlock
        };

  return bilRegisterEncoder(BILPPM_NAME, &encoder);
  }



BILboolean bilPpmUnregisterEncoder()
  {
  return bilUnregisterEncoder(BILPPM_NAME);
  }

