#include "bilRawInternal.h"


typedef struct
  {
  BILdecodeContextStruct parent;
  BILsizei               bytesToDecode;
  BILsizei               bytesDecoded;
  } BILRAWdecodeContextStruct;


static BILcontext bilRawNewDecodeContext()
  {
  BILRAWdecodeContextStruct *context;

  context = malloc(sizeof(BILRAWdecodeContextStruct));
  if (!context)
	{
	bilSetError(BIL_OUT_OF_MEMORY);
	return NULL;
	}

  return (BILcontext)context;
  }


static BILboolean bilRawDeleteDecodeContext(BILcontext _context)
  {
  BILRAWdecodeContextStruct *context = _context;

  assert(context != NULL);

  free(context);

  return BIL_TRUE;
  }


static BILboolean bilRawBeginDecode(BILcontext _context)
  {
  BILRAWdecodeContextStruct *context = _context;
  BILsizei                   width;
  BILsizei                   height;

  assert(context != NULL);

  /* get some image info (must already be set) */
  width  = bilGetImageWidth (context->parent.image);
  height = bilGetImageHeight(context->parent.image);

  /* begin decoding */
  context->bytesToDecode = width * height * bilBytesPerPixel(BIL_RGBA, BIL_UNSIGNED_BYTE);
  context->bytesDecoded  = 0;

  /* clear the image */
  /* memset(context->parent.image->pixels, 0, context->bytesToDecode); */

  return BIL_TRUE;
  }


static BILboolean bilRawEndDecode(BILcontext _context)
  {
  BILRAWdecodeContextStruct *context = _context;

  assert(context != NULL);

  /* make sure we decoded the right number of bytes */
  if (context->bytesDecoded != context->bytesToDecode)
	{
	bilSetError(BIL_INVALID_OPERATION);
	return BIL_FALSE;
	}

  return BIL_TRUE;
  }


static BILsizei bilRawDecodeBlock(BILcontext _context, const BILvoid *block, BILsizei size)
  {
  BILRAWdecodeContextStruct *context = _context;

  assert(context != NULL);
  assert(block   != NULL);
  assert(size    >= 0);

  /* make sure we dont decode too much */
  if (context->bytesDecoded + size > context->bytesToDecode)
	size = context->bytesToDecode - context->bytesDecoded;

  /* decode the block */
  //memcpy(&(context->parent.image->pixels[context->bytesDecoded]), block, size);
  //context->bytesDecoded += size;

  return size;
  }


static BILboolean bilRawValidBlock(BILcontext _context, const BILvoid *block, BILsizei size)
  {
  BILRAWdecodeContextStruct *context = _context;  

  assert(context != NULL);
  assert(block   != NULL);
  assert(size    >= 0);

  /* a zero sized block is NOT valid */
  if (size <= 0)
	return BIL_FALSE;

  return BIL_TRUE;
  }


BILboolean drvRawRegisterDecoder()
  {
  BILdecoderStruct decoder = 
	{
	/* ilVersion           */ BIL_VERSION,
	/* drvVersion          */ BILRAW_VERSION,
	/* newDecodeContext    */ bilRawNewDecodeContext,
	/* deleteDecodeContext */ bilRawDeleteDecodeContext,
    /* beginDecode         */ bilRawBeginDecode,
	/* endDecode           */ bilRawEndDecode,
	/* decodeBlock         */ bilRawDecodeBlock,
	/* validBlock          */ bilRawValidBlock
	};

  return bilRegisterDecoder(BILRAW_NAME, &decoder);
  }