#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <IL/il_internal.h>
#include <IL/drivers/drvRaw.h>

typedef struct
  {
  ILencodeContextStruct parent;
  ILsizei               bytesToEncode;
  ILsizei               bytesEncoded;
  } DRVRAWencodeContextStruct;


static ILcontext drvRawNewEncodeContext()
  {
  DRVRAWencodeContextStruct *context;

  context = malloc(sizeof(DRVRAWencodeContextStruct));
  if (!context)
	{
	ilSetError(IL_OUT_OF_MEMORY);
	return NULL;
	}

  return (ILcontext)context;
  }


static ILboolean drvRawDeleteEncodeContext(ILcontext _context)
  {
  DRVRAWencodeContextStruct *context = _context;

  assert(context != NULL);

  free(context);

  return IL_TRUE;
  }


static ILboolean drvRawBeginEncode(ILcontext _context)
  {
  DRVRAWencodeContextStruct *context = _context;
  ILsizei                    width;
  ILsizei                    height;
  ILsizei                    numChannels;
  ILsizei                    channelDepth;

  assert(context != NULL);

  /* get some image info */
  width        = ilGetWidth(context->parent.image);
  height       = ilGetHeight(context->parent.image);
  numChannels  = ilGetNumChannels(context->parent.image);
  channelDepth = ilGetChannelDepth(context->parent.image);

  /* begin encoding */
  context->bytesToEncode = width * height * numChannels * channelDepth;
  context->bytesEncoded  = 0;

  return IL_TRUE;
  }


static ILboolean drvRawEndEncode(ILcontext _context)
  {
  DRVRAWencodeContextStruct *context = _context;

  assert(context != NULL);

  /* make sure we encoded the right number of bytes */
  if (context->bytesEncoded != context->bytesToEncode)
	{
	ilSetError(IL_INVALID_OPERATION);
	return IL_FALSE;
	}
  
  return IL_TRUE;
  }


static ILsizei drvRawEncodeBlock(ILcontext _context, ILvoid *block, ILsizei size)
  {
  DRVRAWencodeContextStruct *context = _context;

  assert(context != NULL);

  /* make sure we dont encode too much */
  if (context->bytesEncoded + size > context->bytesToEncode)
	size = context->bytesToEncode - context->bytesEncoded;

  /* encode the block */
  memcpy(block, &(context->parent.image->pixels[context->bytesEncoded]), size);
  context->bytesEncoded += size;

  return size;
  }


ILboolean drvRawRegisterEncoder()
  {
  ILencoderStruct encoder = 
	{
	/* ilVersion           */ IL_VERSION,
	/* drvVersion          */ DRVRAW_VERSION,
	/* newEncodeContext    */ drvRawNewEncodeContext,
	/* deleteEncodeContext */ drvRawDeleteEncodeContext,
    /* beginEncode         */ drvRawBeginEncode,
	/* endEncode           */ drvRawEndEncode,
	/* encodeBlock         */ drvRawEncodeBlock
	};

  return ilRegisterEncoder(DRVRAW_NAME, &encoder);
  }
