#ifndef _BIL_H_
#define _BIL_H_

/* begin C function definitions */
#ifdef __cplusplus
extern "C" {
#endif

#if defined(_MSC_VER)
#  pragma warning(disable:4244) /* disable bogus conversion warnings */
#  pragma warning(disable:4305) /* VC++ 5.0 version of above warning */
#  include <crtdbg.h>           /* include memory debugger           */
#endif

/* import/export stuff */
#if defined(WIN32)
#  if defined(BIL_DLL_EXPORT)
#    define BIL_API	__declspec(dllexport)
#  elif defined(BIL_DLL_IMPORT)
#    define BIL_API	__declspec(dllimport)
#  else
#    define BIL_API	extern
#  endif
#else
#  define BIL_API	extern
#endif

/* version stuff */
#define BIL_VERSION_MAKE(major,minor,release) (((major)*10000) + (((minor)%100)*100) + ((release)%100))
#define BIL_VERSION_MAJOR(version)            (((version) / 10000)      )
#define BIL_VERSION_MINOR(version)            (((version) / 100  ) % 100)
#define BIL_VERSION_RELEASE(version)          (((version)        ) % 100)
#define BIL_VERSION                           BIL_VERSION_MAKE(1,0,1)

/* boolean values */
#define BIL_FALSE         (0)
#define BIL_TRUE          (!BIL_FALSE)

/* errors */
typedef enum
  {
  BIL_NO_ERROR           = 0x0000,
  BIL_INVALID_VALUE      = 0x0501,
  BIL_INVALID_ENUM       = 0x0500,
  BIL_INVALID_OPERATION  = 0x0502,
  BIL_STACK_OVERFLOW     = 0x0503,
  BIL_STACK_UNDERFLOW    = 0x0504,
  BIL_OUT_OF_MEMORY      = 0x0505,

  BIL_FILE_WRITE_ERROR   = 0x0570,
  BIL_FILE_READ_ERROR    = 0x0571
  } BILerror;

/* types */
typedef enum
  {
  /* supported image types */
  BIL_UNSIGNED_BYTE      = 0x1401,
  BIL_UNSIGNED_SHORT     = 0x1403,
  BIL_FLOAT              = 0x1406,
  BIL_NUM_TYPES          = 3,

  /* supported parasite types */
  BIL_BOOLEAN            = 0x1420,
  BIL_INT                = 0x1404,
  BIL_DOUBLE             = 0x140A,
  BIL_CHAR_PTR           = 0x1430,
  BIL_VOID_PTR           = 0x1431

  /* unsupported OpenGL types 
  BIL_UNSIGNED_INT       = 0x1405,
  BIL_BYTE               = 0x1400,
  BIL_SHORT              = 0x1402, */
  } BILtype;

/* formats */
typedef enum
  {
  BIL_RGB                = 0x1907,
  BIL_RGBA               = 0x1908,
  BIL_RED                = 0x1903,
  BIL_GREEN              = 0x1904,
  BIL_BLUE               = 0x1905,
  BIL_ALPHA              = 0x1906,
  BIL_LUMINANCE          = 0x1909,
  BIL_LUMINANCE_ALPHA    = 0x190A,
  BIL_NUM_FORMATS        = 8
  } BILformat;


typedef unsigned char  BILboolean;
typedef signed   int   BILsizei;
typedef signed   char  BILbyte;
typedef unsigned char  BILubyte;
typedef signed   short BILshort;
typedef unsigned short BILushort;
typedef signed   int   BILint;
typedef unsigned int   BILuint;
typedef char           BILchar;
typedef float          BILfloat;
typedef double         BILdouble;
typedef void           BILvoid;
typedef void          *BILimage;
typedef void          *BILcontext;

BIL_API BILimage          bilNewImage(BILsizei width, BILsizei height);
BIL_API BILimage          bilCopyImage(BILimage image);
BIL_API BILboolean        bilDeleteImage(BILimage image);
BIL_API BILboolean        bilClearImage(BILimage image);

BIL_API BILboolean        bilResizeImage(BILimage image, BILsizei width, BILsizei height);
BIL_API BILboolean        bilResampleImage(BILimage image, BILsizei width, BILsizei height, const BILchar *filter);
BIL_API BILboolean        bilEnumerateFilters(BILchar *string, BILsizei stringSize);

BIL_API BILvoid          *bilGetImagePtr(BILimage image, BILint x, BILint y);
BIL_API BILsizei          bilGetImageWidth(BILimage image);
BIL_API BILsizei          bilGetImageHeight(BILimage image);
BIL_API BILsizei          bilBytesPerPixel(BILformat format, BILtype type);

BIL_API BILcontext        bilBeginEncode(BILimage image, const BILchar *driver);
BIL_API BILsizei          bilEncodeBlock(BILcontext context, BILvoid *block, BILsizei size);
BIL_API BILboolean        bilEndEncode(BILcontext context);

BIL_API BILcontext        bilBeginDecode(BILimage image, const BILchar *driver);
BIL_API BILboolean        bilValidBlock(BILcontext context, const BILvoid *block, BILsizei size);
BIL_API BILsizei          bilDecodeBlock(BILcontext context, const BILvoid *block, BILsizei size);
BIL_API BILboolean        bilEndDecode(BILcontext context);

BIL_API BILboolean        bilSaveImage(BILimage *image, const BILchar *fileName, const BILchar *driver);
BIL_API BILboolean        bilLoadImage(BILimage *image, const BILchar *fileName, const BILchar *driver);

BIL_API BILboolean        bilSetImageParasiteBoolean(BILimage image, const BILchar *key, const BILboolean *values, BILsizei count);
BIL_API BILboolean        bilSetImageParasiteInt    (BILimage image, const BILchar *key, const BILint     *values, BILsizei count);
BIL_API BILboolean        bilSetImageParasiteDouble (BILimage image, const BILchar *key, const BILdouble  *values, BILsizei count);
BIL_API BILboolean        bilSetImageParasiteCharPtr(BILimage image, const BILchar *key, const BILchar    *value);
BIL_API BILboolean        bilSetImageParasiteVoidPtr(BILimage image, const BILchar *key, const BILvoid    *value,  BILsizei size);

BIL_API const BILboolean *bilGetImageParasiteBoolean(BILimage image, const BILchar *key);
BIL_API const BILint     *bilGetImageParasiteInt    (BILimage image, const BILchar *key);
BIL_API const BILdouble  *bilGetImageParasiteDouble (BILimage image, const BILchar *key);
BIL_API const BILchar    *bilGetImageParasiteCharPtr(BILimage image, const BILchar *key);
BIL_API const BILvoid    *bilGetImageParasiteVoidPtr(BILimage image, const BILchar *key);

BIL_API BILsizei          bilGetImageParasiteCount(BILimage image, const BILchar *key);
BIL_API BILsizei          bilGetImageParasiteSize(BILimage image, const BILchar *key);
BIL_API BILtype           bilGetImageParasiteType(BILimage image, const BILchar *key);
BIL_API BILboolean        bilEnumerateImageParasites(BILimage image, BILchar *string, BILsizei stringSize);
BIL_API BILboolean        bilCopyImageParasites(BILimage src, BILimage dst);

BIL_API BILboolean        bilDrawPixels(BILimage image, BILint x, BILint y, BILsizei width, BILsizei height, BILformat format, BILtype type, const BILvoid *pixels);
BIL_API BILboolean        bilReadPixels(BILimage image, BILint x, BILint y, BILsizei width, BILsizei height, BILformat format, BILtype type,       BILvoid *pixels);
BIL_API BILboolean        bilCopyPixels(BILimage srcImage, BILint srcX, BILint srcY, BILsizei width, BILsizei height, BILimage dstImage, BILint dstX, BILint dstY);

BIL_API BILboolean        bilFlipImageHorizontal(BILimage image);
BIL_API BILboolean        bilFlipImageVertical(BILimage image);

BIL_API BILboolean        bilEnumerateEncoders(BILchar *string, BILsizei stringSize);
BIL_API BILboolean        bilEnumerateDecoders(BILchar *string, BILsizei stringSize);

BIL_API const BILchar    *bilNextKey(const BILchar *string, BILchar *key, BILsizei keySize);
BIL_API BILboolean        bilContainsKey(const BILchar *string, const BILchar *key);

BIL_API BILerror          bilGetError();

/* end C function definitions */
#ifdef __cplusplus
    }
#endif

#endif /* _BIL_H_ */