#ifndef _BIL_DRIVER_H_
#define _BIL_DRIVER_H_

/* begin C function definitions */
#ifdef __cplusplus
extern "C" {
#endif

/* include various sub-include files */
#include <bil/bil.h>

/* just for some error checking */
#define BIL_ENCODE_SIGNATURE  (0x21434E45)
#define BIL_DECODE_SIGNATURE  (0x21434544)

/* our own assert statements */
#define bilAssert(CONDITION, FAIL_CODE) \
  do {                                  \
    assert(CONDITION);                  \
    if (!(CONDITION))                   \
      { FAIL_CODE; }                    \
  } while(0)

#define bilCheckParameter(CONDITION, FAIL_CODE) \
  bilAssert(CONDITION, bilSetError(BIL_INVALID_ENUM); FAIL_CODE)

typedef struct
  {
  BILchar *key;
  BILtype  type;
  BILsizei count;
  } BILparameterStruct;

typedef struct
  {
  BILuint             bilVersion;
  BILuint             version;
  BILparameterStruct *parameters;
  BILcontext        (*newEncodeContext)();
  BILboolean        (*deleteEncodeContext)(BILcontext context);
  BILboolean        (*beginEncode)(BILcontext context);
  BILboolean        (*endEncode)(BILcontext context);
  BILsizei          (*encodeBlock)(BILcontext context, BILvoid *block, BILsizei size);
  } BILencoderStruct;

typedef struct
  {
  BILuint             bilVersion;
  BILuint             version;
  BILparameterStruct *parameters;
  BILcontext        (*newDecodeContext)();
  BILboolean        (*deleteDecodeContext)(BILcontext context);
  BILboolean        (*beginDecode)(BILcontext context);
  BILboolean        (*endDecode)(BILcontext context);
  BILsizei          (*decodeBlock)(BILcontext context, const BILvoid *block, BILsizei size);
  BILboolean        (*validBlock)(BILcontext context, const BILvoid *block, BILsizei size);
  } BILdecoderStruct;

typedef struct
  {
  BILuint             bilVersion;
  BILuint             version;
  BILparameterStruct *parameters;
  BILdouble           supp;
  BILdouble           blur;
  BILboolean          window;
  BILboolean          cardinal;
  BILboolean          unitrange;
  BILvoid            *data;
  BILdouble         (*function)(BILdouble x, BILvoid *data);
  BILvoid           (*init)(BILdouble a, BILdouble b, BILvoid *data);
  } BILfilterStruct;

typedef struct
  {
  BILuint                 signature;
  BILimage                image;
  const BILencoderStruct *encoder;
  } BILencodeContextStruct;

typedef struct
  {
  BILuint                 signature;
  BILimage                image;
  const BILdecoderStruct *decoder;
  } BILdecodeContextStruct;

typedef BILboolean (*BILregisterDriverFunction)();

BIL_API BILboolean              bilRegisterEncoder(const BILchar *name, const BILencoderStruct *encoder);
BIL_API BILboolean              bilUnregisterEncoder(const BILchar *name);
BIL_API const BILencoderStruct *bilGetEncoder(const BILchar *name);

BIL_API BILboolean              bilRegisterDecoder(const BILchar *name, const BILdecoderStruct *decoder);
BIL_API BILboolean              bilUnregisterDecoder(const BILchar *name);
BIL_API const BILdecoderStruct *bilGetDecoder(const BILchar *name);

BIL_API BILboolean              bilRegisterFilter(const BILchar *name, const BILfilterStruct *filter);
BIL_API BILboolean              bilUnregisterFilter(const BILchar *name);
BIL_API const BILfilterStruct  *bilGetFilter(const BILchar *name);

BIL_API BILvoid                *bilMalloc(BILsizei size);
BIL_API BILvoid                 bilFree(BILvoid *pointer);
BIL_API BILvoid                *bilRealloc(BILvoid *pointer, BILsizei size);
BIL_API BILvoid                *bilMemdup(const BILvoid *pointer, BILsizei size);

BIL_API void                    bilSetError(BILerror error);

/* end C function definitions */
#ifdef __cplusplus
    }
#endif

#endif /* _BIL_DRIVER_H_ */