#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#if WIN32
#  include <io.h>
#endif
#include <fcntl.h>
#include <bil/bil.h>
#include <bil/bilJpeg.h>
#include <bil/bilPng.h>
#include <bil/bilPpm.h>


void checkError(const BILchar *where)
  {
  switch (bilGetError())
	{
	case BIL_NO_ERROR:          return;
	case BIL_INVALID_VALUE:     fprintf(stderr, "BIL error: %s: BIL_INVALID_VALUE\n", where);     exit(1);
	case BIL_INVALID_ENUM:	    fprintf(stderr, "BIL error: %s: BIL_INVALID_ENUM\n", where);      exit(1);
	case BIL_INVALID_OPERATION: fprintf(stderr, "BIL error: %s: BIL_INVALID_OPERATION\n", where); exit(1);
	case BIL_STACK_OVERFLOW:    fprintf(stderr, "BIL error: %s: BIL_STACK_OVERFLOW\n", where);    exit(1);
	case BIL_STACK_UNDERFLOW:   fprintf(stderr, "BIL error: %s: BIL_STACK_UNDERFLOW\n", where);   exit(1);
	case BIL_OUT_OF_MEMORY:     fprintf(stderr, "BIL error: %s: BIL_OUT_OF_MEMORY\n", where);     exit(1);
	case BIL_FILE_WRITE_ERROR:  fprintf(stderr, "BIL error: %s: BIL_FILE_WRITE_ERROR\n", where);  exit(1);
	case BIL_FILE_READ_ERROR:   fprintf(stderr, "BIL error: %s: BIL_FILE_READ_ERROR\n", where);   exit(1);
	}

  fprintf(stderr, "BIL error: %s: Unknown error\n", where); 
  exit(1);
  }


void loadImage(BILimage image, FILE *file, const BILchar *decoder)
  {
  BILubyte   buffer[1024];
  BILsizei   size;
  BILcontext context;

  context = bilBeginDecode(image, decoder);
  checkError("bilBeginDecode");

  size = fread(&buffer, 1, sizeof(buffer), file);
  while (bilValidBlock(context, &buffer, size))
    {
    bilDecodeBlock(context, &buffer, size);
    size = fread(&buffer, 1, sizeof(buffer), file);
    }

  bilEndDecode(context);
  checkError("bilEndDecode");
  }


void saveImage(BILimage image, FILE *file, const BILchar *encoder)
  {
  BILubyte   buffer[1024];
  BILsizei   size;
  BILcontext context;

  context = bilBeginEncode(image, encoder);
  checkError("bilBeginEncode");
  
  while ((size = bilEncodeBlock(context, &buffer, sizeof(buffer))) > 0)
    fwrite(&buffer, size, 1, file);

  bilEndEncode(context);
  checkError("bilEndEncode");
  }


int main(int argc, char *argv[])
  {
  BILimage    image;
  FILE       *inFile;
  FILE       *outFile;
  BILchar    *decoder = "jpeg";
  BILchar    *encoder = "jpeg";

#ifdef _MSC_VER
  _CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF|_CRTDBG_CHECK_ALWAYS_DF|_CRTDBG_LEAK_CHECK_DF);
#endif
  
  /* register drivers */
  bilJpegRegisterDecoder();
  bilJpegRegisterEncoder();
  bilPngRegisterDecoder();
  bilPngRegisterEncoder();
  bilPpmRegisterDecoder();
  bilPpmRegisterEncoder();

  //inFile  = stdin;  //fopen("c:/stripinis_04.jpg", "rb");
  //outFile = stdout; //fopen("c:/output.jpg", "wb");

  inFile  = fopen("d:/prg/bil/VC98/translate/Release/earthlights_dmsp_big.jpg", "rb");
  outFile = fopen("d:/prg/bil/VC98/translate/Release/test.png", "wb");

  if (argc > 1) decoder = argv[1]; //decoder="png";
  if (argc > 2) encoder = argv[2]; //encoder="png";

#ifdef WIN32
  /* open input/output files */
  setmode(fileno(inFile),  O_BINARY);
  setmode(fileno(outFile), O_BINARY);
#endif

  /* create image */
  image = bilNewImage(0, 0);
  checkError("bilNewImage");

  /* convert... */
  loadImage(image, inFile,  decoder);

  /* save image */
  saveImage(image, outFile, encoder);

  /* delete image */
  bilDeleteImage(image);
  checkError("bilDeleteImage");

  return 0;
  }
